package com.ruoyi.system.controller.user;

import com.ruoyi.system.api.domain.SysDept;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.validation.annotation.Validated;
import java.util.function.Function;
import com.ruoyi.system.api.domain.SysRole;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import org.springframework.web.bind.annotation.RequestBody;
import com.ruoyi.common.security.annotation.InnerAuth;
import java.util.Set;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.common.core.domain.R;
import org.springframework.web.bind.annotation.PathVariable;
import java.io.IOException;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.bind.annotation.PostMapping;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import javax.servlet.http.HttpServletResponse;
import com.ruoyi.common.core.web.domain.AjaxResult;
import org.springframework.web.bind.annotation.GetMapping;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import java.util.List;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysPermissionService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.web.controller.BaseController;

@RestController
@RequestMapping({ "/user" })
public class SysUserController extends BaseController
{
    @Autowired
    private ISysUserService userService;
    @Autowired
    private ISysRoleService roleService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private ISysPostService postService;
    @Autowired
    private ISysPermissionService permissionService;
    @Autowired
    private ISysConfigService configService;
    
    @RequiresPermissions({ "system:user:list" })
    @GetMapping({ "/list" })
    public TableDataInfo list(final SysUser user) {
        this.startPage();
        final List<SysUser> list = this.userService.selectUserList(user);
        return this.getDataTable((List)list);
    }
    
    @GetMapping({ "/selectList" })
    public AjaxResult selectList(final SysUser user) {
        return AjaxResult.success((Object)this.userService.selectList(user));
    }
    
    @Log(title = "用户管理", businessType = BusinessType.EXPORT)
    @RequiresPermissions({ "system:user:export" })
    @PostMapping({ "/export" })
    public void export(final HttpServletResponse response, final SysUser user) {
        final List<SysUser> list = this.userService.selectUserList(user);
        final ExcelUtil<SysUser> util = (ExcelUtil<SysUser>)new ExcelUtil((Class)SysUser.class);
        util.exportExcel(response, (List)list, "用户数据");
    }
    
    @Log(title = "用户管理", businessType = BusinessType.IMPORT)
    @RequiresPermissions({ "system:user:import" })
    @PostMapping({ "/importData" })
    public AjaxResult importData(final MultipartFile file, final boolean updateSupport) throws Exception {
        final ExcelUtil<SysUser> util = (ExcelUtil<SysUser>)new ExcelUtil((Class)SysUser.class);
        final List<SysUser> userList = util.importExcel(file.getInputStream());
        final String operName = SecurityUtils.getUsername();
        final String message = this.userService.importUser((List)userList, Boolean.valueOf(updateSupport), operName);
        return this.success(message);
    }
    
    @PostMapping({ "/importTemplate" })
    public void importTemplate(final HttpServletResponse response) throws IOException {
        final ExcelUtil<SysUser> util = (ExcelUtil<SysUser>)new ExcelUtil((Class)SysUser.class);
        util.importTemplateExcel(response, "用户数据");
    }
    
    @InnerAuth
    @GetMapping({ "/info/{username}" })
    public R<LoginUser> info(@PathVariable("username") final String username) {
        final SysUser sysUser = this.userService.selectUserByUserName(username);
        if (StringUtils.isNull((Object)sysUser)) {
            return (R<LoginUser>)R.fail("用户名或密码错误");
        }
        final Set<String> roles = this.permissionService.getRolePermission(sysUser);
        final Set<String> permissions = this.permissionService.getMenuPermission(sysUser);
        final LoginUser sysUserVo = new LoginUser();
        sysUserVo.setSysUser(sysUser);
        sysUserVo.setRoles((Set)roles);
        sysUserVo.setPermissions((Set)permissions);
        return (R<LoginUser>)R.ok((Object)sysUserVo);
    }
    
    @InnerAuth
    @PostMapping({ "/register" })
    public R<Boolean> register(@RequestBody final SysUser sysUser) {
        final String username = sysUser.getUserName();
        if (!"true".equals(this.configService.selectConfigByKey("sys.account.registerUser"))) {
            return (R<Boolean>)R.fail("当前系统没有开启注册功能！");
        }
        if (!this.userService.checkUserNameUnique(sysUser)) {
            return (R<Boolean>)R.fail("保存用户'" + username + "'失败，注册账号已存在");
        }
        return (R<Boolean>)R.ok((Object)this.userService.registerUser(sysUser));
    }
    
    @GetMapping({ "getInfo" })
    public AjaxResult getInfo() {
        final SysUser user = this.userService.selectUserById(SecurityUtils.getUserId());
        final Set<String> roles = this.permissionService.getRolePermission(user);
        final Set<String> permissions = this.permissionService.getMenuPermission(user);
        final AjaxResult ajax = AjaxResult.success();
        ajax.put("user", (Object)user);
        ajax.put("roles", (Object)roles);
        ajax.put("permissions", (Object)permissions);
        return ajax;
    }
    
    @RequiresPermissions({ "system:user:query" })
    @GetMapping({ "/", "/{userId}" })
    public AjaxResult getInfo(@PathVariable(value = "userId", required = false) final Long userId) {
        this.userService.checkUserDataScope(userId);
        final AjaxResult ajax = AjaxResult.success();
        final List<SysRole> roles = this.roleService.selectRoleAll();
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect((Collector<? super Object, ?, List<? super Object>>)Collectors.toList()));
        ajax.put("posts", (Object)this.postService.selectPostAll());
        if (StringUtils.isNotNull((Object)userId)) {
            final SysUser sysUser = this.userService.selectUserById(userId);
            ajax.put("data", (Object)sysUser);
            ajax.put("postIds", (Object)this.postService.selectPostListByUserId(userId));
            ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
        }
        return ajax;
    }
    
    @RequiresPermissions({ "system:user:add" })
    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody final SysUser user) {
        this.deptService.checkDeptDataScope(user.getDeptId());
        this.roleService.checkRoleDataScope(user.getRoleIds());
        if (!this.userService.checkUserNameUnique(user)) {
            return this.error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
            return this.error("新增用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
            return this.error("新增用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setCreateBy(SecurityUtils.getUserId().toString());
        user.setCreateName(SecurityUtils.getLoginUser().getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        return this.toAjax(this.userService.insertUser(user));
    }
    
    @RequiresPermissions({ "system:user:edit" })
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody final SysUser user) {
        this.userService.checkUserAllowed(user);
        this.userService.checkUserDataScope(user.getUserId());
        this.deptService.checkDeptDataScope(user.getDeptId());
        this.roleService.checkRoleDataScope(user.getRoleIds());
        if (!this.userService.checkUserNameUnique(user)) {
            return this.error("修改用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
            return this.error("修改用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
            return this.error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.userService.updateUser(user));
    }
    
    @RequiresPermissions({ "system:user:remove" })
    @Log(title = "用户管理", businessType = BusinessType.DELETE)
    @DeleteMapping({ "/{userIds}" })
    public AjaxResult remove(@PathVariable final Long[] userIds) {
        if (ArrayUtils.contains(userIds, SecurityUtils.getUserId())) {
            return this.error("当前用户不能删除");
        }
        return this.toAjax(this.userService.deleteUserByIds(userIds));
    }
    
    @RequiresPermissions({ "system:user:resetPwd" })
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping({ "/resetPwd" })
    public AjaxResult resetPwd(@RequestBody final SysUser user) {
        this.userService.checkUserAllowed(user);
        this.userService.checkUserDataScope(user.getUserId());
        user.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.userService.resetPwd(user));
    }
    
    @RequiresPermissions({ "system:user:resetPwd" })
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping({ "/changeStatus" })
    public AjaxResult changeStatus(@RequestBody final SysUser user) {
        this.userService.checkUserAllowed(user);
        this.userService.checkUserDataScope(user.getUserId());
        user.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.userService.updateUserStatus(user));
    }
    
    @RequiresPermissions({ "system:user:query" })
    @GetMapping({ "/authRole/{userId}" })
    public AjaxResult authRole(@PathVariable("userId") final Long userId) {
        final AjaxResult ajax = AjaxResult.success();
        final SysUser user = this.userService.selectUserById(userId);
        final List<SysRole> roles = this.roleService.selectRolesByUserId(userId);
        ajax.put("user", (Object)user);
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect((Collector<? super Object, ?, List<? super Object>>)Collectors.toList()));
        return ajax;
    }
    
    @RequiresPermissions({ "system:user:edit" })
    @Log(title = "用户管理", businessType = BusinessType.GRANT)
    @PutMapping({ "/authRole" })
    public AjaxResult insertAuthRole(final Long userId, final Long[] roleIds) {
        this.userService.checkUserDataScope(userId);
        this.roleService.checkRoleDataScope(roleIds);
        this.userService.insertUserAuth(userId, roleIds);
        return this.success();
    }
    
    @RequiresPermissions({ "system:user:list" })
    @GetMapping({ "/deptTree" })
    public AjaxResult deptTree(final SysDept dept) {
        return this.success((Object)this.deptService.selectDeptTreeList(dept));
    }
}
